% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Ph.D. Course "Cognitive CPSs: from theory to practice"
% Prof. Manuel Roveri, Prof. Cesare Alippi
% Politecnico di Milano, Italy
% March 2016
% 
% ----- "From Metrology to digital data"
% -> Example 2_A) ACCURACY 
%   
% Exercise:
% -> Change the Number of Samples to see the effects on the estimated bias
%   and the calibrated measurements
% -> See the effects of the standard deviation of the noise
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Consider the additive signal plus noise model x = x0 + eta
% where x0 is the deal, noise free unknown measurement value 
% and eta is an i.i.d. zero mean random variable

clc;
clear all;

x0 = 10;        % Ideal, noise free unknown value
n = 1000;        % Number of acquired samples
bias = 1;       % Bias value associated with the measure
std_noise = 10;  % Standar deviation of the noise

% Generated samples with Gaussian noise
X = x0 + bias + std_noise * randn(n,1);

figure, hist(X,100);
title('Acquired measurements')
line ([x0,x0],[0,max(hist(X,100))],'Color','red');
% Sample mean to estimate the expected value
Mean  = mean(X);
fprintf('Sample mean = %.2f\n',Mean);


% How to calibrate a sensor? 
% x0_calibration is assumed to be known
x0_calibration = 0;
X_calibration = x0_calibration + bias + std_noise * randn(n,1);

% estimated the bias in the measurements
estimated_bias = mean(X_calibration) - x0_calibration;

X = x0 + bias + std_noise * randn(n,1);

% calibrate the measurements
Calibrated_X = X - estimated_bias;

figure, hist(Calibrated_X,100);
title('Calibrated measurements')
line ([x0,x0],[0,max(hist(Calibrated_X,100))],'Color','red');

Mean_Calibrated  = mean(Calibrated_X);
fprintf('Estimated bias = %.2f\n',estimated_bias);
fprintf('Calibrated Sample mean = %.2f\n',Mean_Calibrated);







