function [tsInit , tsEnd , tsLengths , testDetectingChange, originalDetections , tsBefore] = f_ICI_test_MultiChange(DataSet , tsLength_init , params)
%
% function [tsInit , tsEnd , tsLengths , testDetectingChange, originalDetections , tsBefore] = f_ICI_test_MultiChange(DataSet , tsLength_init , params)
%
% tsLength_init  rappresenta la lunghezza del training set iniziale
%
% params  una struttura contenente
% params.windowWidth
% params.Gamma
% params.changePoint ,
% params.do_adaptiveGamma
% params.do_show
% params.do_debug_duro
% params.K_refinement
% params.MinimumTS_Size
% params.weights
%
%
% output
%
% tsInit                           - vector of refined change detection estimates (i.e T_{ref})
% tsEnd                          - vector of detected changes (i.e \hat{T})
% tsLengths                   - vector lengts of each trainig set (i.e for each change \hat{T} - T_{ref})
% testDetectingChange - struttura contente l'indice del test che ha beccato il cambiamento
%                                    - testDetectingChange.Init     test che ha beccato \hat{T}
%                                    - testDetectingChange.Refined  test che ha beccato T_{ref}
%originalDetections -  initial detections, without imposing minimum TS size.
%
%
% please cite this work as
% [1] Cesare Alippi, Giacomo Boracchi and Manuel Roveri,
% "Change Detection Tests Using the ICI rule"
% in Proceedings of IJCNN 2010, the International Joint Conference on Neural Networks 18 - 23 July, 2010 Barcelona, Spain.
%
% [2] Cesare Alippi, Giacomo Boracchi and Manuel Roveri,
% "Adaptive Classifiers with ICI-based Adaptive Knowledge Base Management"
% ICANN 2010, 20th International Conference on Artificial Neural Networks, September 15-18, 2010, Thessaloniki, Greece,
% Lecture Notes on Computer Science. Springer Berlin / Heidelberg, vol 6353, pg 458-467
%
% [3] Cesare Alippi, Giacomo Boracchi and Manuel Roveri,
% "A just-in-time adaptive classification system based on the intersection of confidence intervals rule"
%  Neural Networks, Elsevier vol. 24 (2011), pp. 791-800 (doi: 10.1016/j.neunet.2011.05.012)
%
%
% Giacomo Boracchi
% Politecnico di Milano
% giacomo.boracchi@polimi.it
%
%
% Revision History
% January 2010 - first release
% October 2010 -   added LPA estimates for both features
% December 2010   - changed input: passing test parameter structure
% January 2012       - added the field originalDetections to store the initial detected change, without imposing the minimum TS size - this is to be used for JIT
%                               classifier
% April 2012 - Added CDT on the reversed dataset once the change has been detected to estimate tsBefore

detected = 1;
counter = 1;
testDetectingChange = [];
dimensionDetected =[];

DataSetReduced = DataSet;
DataSetStartIndex = 0;

tsInit = 1;
tsEnd = tsLength_init;
tsLengths = tsLength_init;
originalDetections = tsLength_init;
tsBefore = 1;

% change detection is iterated untill no change is detected (i.e. we reached the end of DataSet)
while(detected(counter) > 0)
    
    counter = counter + 1;
    
    % detect change on DataSetReduced using the trainig set identified
    % before
    [detected(counter) , testDetectingChangeInit(counter) , Theta0 , dimensionDetected(counter)] = f_ICI_test_MultiDim(DataSetReduced , tsLengths(counter - 1) , params);
    
    if detected(counter) > 0
        % run test to identify new Training set
        [refined(counter) , testDetectingChange_temp] = f_ICI_refine_test(DataSetReduced , detected(counter) , dimensionDetected(counter) , tsLengths(counter - 1) , params);
        
        testDetectingChangeRefinement(counter) = testDetectingChange_temp(max([length(testDetectingChange_temp) - 1 , 1]));
        
        % determines the length of the trainingSet, as the distance between the detection from the CDT and the refinement procedure
        tsLengths(counter) = ceil((detected(counter) - refined(counter)) / params.windowWidth) * params.windowWidth;
        tsLength_original(counter) = tsLengths(counter);
		% we set a minimum number of training samples
        tsLengths(counter) = max([tsLengths(counter) , params.MinimumTS_Size]);
        
        % the new training set starts here
        tsInit(counter) = DataSetStartIndex + refined(counter);
        
        originalDetections(counter) = detected(counter);
        
        % and the new training set ends here (we impose a minimum training set size of params.MinimumTS_Size)
        tsEnd(counter) = min([tsInit(counter) + tsLengths(counter) , size(DataSet , 2)]);
        
        % update DataSet and TrainigSet
        DataSetStartIndex = tsInit(counter);
        DataSetReduced = DataSet(: , DataSetStartIndex + 1 : end);
        
        if (length(DataSetReduced) <=  tsLengths(counter) ||  length(DataSetReduced) == 0)
            detected(counter) = 0;
            %warning('Sollevo un falso negativo perch il dataset corrisponde al ts')
        end
        
        %% run the refinement procedure on the reversed datataset, used to determine T_ref2 an early estimate of T*
        % it follows T_before < T^* < T_ref < \hat{T} , tsBefore < T^* < tsInit < tsEnd
        if (detected(counter) > 0)
            reversedDS =  DataSet(: , tsEnd(counter) : - 1 : tsEnd(counter - 1) + 1); % vai dall'ultimo \hat{T} al \hat{T} precedente
            reversedTsLength = tsLengths(counter);
            paramsReversed = params;
            paramsReversed.Gamma = params.GammaReverseAnalysis;
            [reversedRefined(counter)] = f_ICI_test_MultiDim(reversedDS ,reversedTsLength  , paramsReversed);
            
            %   reversedDetection = size(reversedDS , 2); % ripercorri tutto il dataset al contrario, non serve specificare punti di arresto, il datasetReversed  gi a posto
            % [reversedRefined(counter)] = f_ICI_refine_test(reversedDS , reversedDetection , dimensionDetected(counter) , reversedTsLength , params , params.GammaRefinement);
            
            if reversedRefined(counter) == 0
                        tsBefore(counter) = tsEnd(counter) - reversedTsLength; % i.e. tsInit(counter) 
            else
                        tsBefore(counter) = max([tsEnd(counter) - reversedRefined(counter), tsEnd(counter - 1)]);
            end
            
            %disp(['tsInit( - 1) : ' , num2str(tsInit(counter - 1)) , ' tsBefore : ' , num2str(tsBefore(counter)) , ' tsInit : ' , num2str(tsInit(counter)) , ' tsEnd : ' , num2str(tsEnd(counter))])
            if (params.do_show)
                disp(['Concept ' , num2str((counter - 1)) , ' : [ ' , num2str(tsInit(counter - 1))  , ' , ' , num2str(tsBefore(counter)) , ' ] vs Concept ' , num2str(counter) , ' : [ ' , num2str(tsInit(counter)) , ' , ' , num2str(tsEnd(counter)) ,' ]'])
            end
        else
            tsBefore(counter) = tsEnd(counter - 1);
        end
    end
    
    % replace 0s with 1
    tsBefore = max([tsBefore ; ones(size(tsBefore))]);
end

% controllo se per caso non ho avuto una detection proprio all'ultimo campione per cui non posso deifinire il ts
if tsEnd(end) == length(DataSet)
    tsInit = tsInit(1 : end - 1);
    tsEnd = tsEnd(1 : end - 1);
    tsLengths = tsLengths(1 : end - 1);
    tsBefore =  tsBefore(1 : end - 1);
    testDetectingChange = testDetectingChange(1 : end - 1);
    originalDetections = originalDetections(1 : end - 1);
end

if exist('testDetectingChangeRefinement' , 'var')
    testDetectingChange.Init = testDetectingChangeInit;
    testDetectingChange.Refined= testDetectingChangeRefinement;
end


